unit CustomDrawProc;

interface

uses GDIPOBJ, GDIPAPI, GDIPUTIL, Classes;

type
  TSplashCustomEvent = class
  private
    FGPRect: TGPRectF;
    FGPStrForm: TGPStringFormat;
    FGPBrush, FGPBrushShadow: TGPSolidBrush;
    FGPFont: TGPFont;
    FGPImage: TGPBitmap;

    function GetString(CurrStr: integer): string;
    procedure SetAlphaValue(GPBitmap: TGPBitmap);
  public
    constructor Create;
    destructor Destroy; override;
    procedure DrawText(Sender: TObject; GPGraphics: TGPGraphics);
    procedure KeyPress(Sender: TObject; var Key: char);
    procedure DestroySplash(Sender: TObject);
    procedure EndFade(Sender: TObject);
  end;

var
  SplashCustomEvent: TSplashCustomEvent;
  CurrentStr: Integer;
  ArrayStr: array [0..17] of string = (
     '',
     ' ',
     ' ',
     ' ',
     ' ',
     '  ',
     '',
     ' COM',
     '  ',
     '  ',
     ' ',
     ' MUI',
     ' ',
     ' ',
     ' ',
     '  ',
     '',
     ' '
    );

implementation

uses ASFMain, Graphics, Windows, ASFDemoMain;

var
  oldCurrentStr: Integer;
  CurrentStep: Integer;


constructor TSplashCustomEvent.Create;
begin
  FGPFont := TGPFont.Create('Verdana', 11, FontStyleBold);
  FGPBrush := TGPSolidBrush.Create(GetGPColor(clBlue, $FF));
  FGPBrushShadow := TGPSolidBrush.Create(GetGPColor(clLtGray, $FF));
  FGPStrForm := TGPStringFormat.Create;
  FGPImage := TGPBitmap.Create(283, 50, PixelFormat32bppARGB);
  FGPRect := MakeRect(0, 0, FGPImage.GetWidth, FGPImage.GetHeight);
  FGPStrForm.SetAlignment(StringAlignmentCenter);
  FGPStrForm.SetLineAlignment(StringAlignmentNear);
  inherited;
end;

destructor TSplashCustomEvent.Destroy;
begin
  inherited;
  FGPStrForm.Free;
  FGPBrush.Free;
  FGPBrushShadow.Free;
  FGPFont.Free;
  FGPImage.Free;
end;

procedure TSplashCustomEvent.DrawText(Sender: TObject; GPGraphics: TGPGraphics);
var
  s: string;
  tmpGPGraphics: TGPGraphics;
begin
  if AnimationSplashForm <> nil then begin
    //     
    FGPImage.SetResolution(GPGraphics.GetDpiX, GPGraphics.GetDpiY);
    tmpGPGraphics := TGPGraphics.Create(FGPImage);
    tmpGPGraphics.Clear($00FFFFFF);
    try
      s := GetString(CurrentStr);
      ASFDemoMain.EndItemAnimation := CurrentStep >= 13;
      if not ASFDemoMain.EndItemAnimation then
        Inc(CurrentStep);
      if oldCurrentStr <> CurrentStr then begin
        CurrentStep := 0;
        oldCurrentStr := CurrentStr;
      end;
      tmpGPGraphics.SetTextRenderingHint(TextRenderingHintAntiAliasGridFit);
      //  
      tmpGPGraphics.DrawString(s, Length(s), FGPFont,
          MakeRect(FGPRect.X + 1, FGPRect.Y + 1, FGPRect.Width, FGPRect.Height),
          FGPStrForm, FGPBrushShadow);
      //  
      tmpGPGraphics.DrawString(s, Length(s), FGPFont, FGPRect, FGPStrForm, FGPBrush);
      //  
      SetAlphaValue(FGPImage);
      //        
      GPGraphics.DrawImage(FGPImage, 25, 271, 0, 5 + CurrentStep, FGPImage.GetWidth,
          32, UnitPixel);
    finally
      tmpGPGraphics.Free;
    end;
  end;
end;

function TSplashCustomEvent.GetString(CurrStr: integer): string;
begin
  if CurrStr - 1 < 0 then
    Result := '' + #13#10
  else
    Result := ArrayStr[CurrStr - 1] + #13#10;
  if CurrStr >= Length(ArrayStr) then
    Result := Result + '' + #13#10
  else
    Result := Result + ArrayStr[CurrStr] + #13#10;
  if CurrStr + 1 >= Length(ArrayStr) then begin
    Result := Result + '' + #13#10 + '';
    Exit;
  end
  else
    Result := Result + ArrayStr[CurrStr + 1] + #13#10;
  if CurrStr + 2 >= Length(ArrayStr) then
    Result := Result + ''
  else
    Result := Result + ArrayStr[CurrStr + 2];
end;

procedure TSplashCustomEvent.SetAlphaValue(GPBitmap: TGPBitmap);
var
  Width, Height, Row, Column: Integer;
  color, colorTemp: TGPColor;
begin
  Width  := GPBitmap.GetWidth;
  Height := GPBitmap.GetHeight;
  for Row := 0 to Height - 1 do
  begin
    for Column := 0 to Width - 1 do
    begin
      GPBitmap.GetPixel(Column, Row, color);
      colorTemp := color;
      if (Row >= 5 + CurrentStep) and (Row < 15 + CurrentStep) and (GetAlpha(color) <> 0) then
        colorTemp := MakeColor((255 * (Row - (5 + CurrentStep)) div 10),
           GetRed(color), GetGreen(color), GetBlue(color));
      if (Row >= 26 + CurrentStep) and (Row < 37 + CurrentStep) and (GetAlpha(color) <> 0) then
        colorTemp := MakeColor((255 - (255 * (Row - (26 + CurrentStep))  div 11)),
           GetRed(color), GetGreen(color), GetBlue(color));
      GPBitmap.SetPixel(Column, Row, colorTemp);
    end;
  end;
end;

procedure TSplashCustomEvent.KeyPress(Sender: TObject; var Key: char);
begin
  if (Ord(key) = vk_Escape) and (AnimationSplashForm <> nil) then begin
    AnimationSplashForm.FadeSpeed := 255; //   
    AnimationSplashForm.CloseSplash(0, True);
  end;
end;

procedure TSplashCustomEvent.DestroySplash(Sender: TObject);
begin
  AnimationSplashForm := nil;  //    AV
end;

procedure TSplashCustomEvent.EndFade(Sender: TObject);
begin
  ASFDemoMain.EndFadeSplash := True;
end;



initialization
  SplashCustomEvent := TSplashCustomEvent.Create;

finalization
  SplashCustomEvent.Free;

end.
