unit CustomEvents;

interface

uses GDIPOBJ, GDIPAPI, GDIPUTIL, ASFMain, Classes, Controls, WaveEng, Registry;

type
  TSplashCustomEvent = class
  private
    FGPImage: TGPBitmap;
    FWater: TWave;
    FStep: Integer;
    FPoolLeft: Integer;
    FPoolTop: Integer;
    FRain: Boolean;
    function PixelInRect(x, y: integer; aRect: TGPRectF): boolean; overload;
    function PixelInRect(x, y: integer; aRect: TGPRect): Boolean; overload;
    function InPool(X, Y: Integer): Boolean;
    procedure SetRainTextData(ASF: TAnimationSplashForm);
  public
    crArrow: TCursor;
    crHand: TCursor;
    AboutShow: Boolean;
    constructor Create;
    destructor Destroy; override;
    procedure CustomDraw(Sender: TObject; GPGraphics: TGPGraphics);
    procedure MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure DestroySplash(Sender: TObject);
    procedure MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X,
        Y: Integer);

  end;

  TClickLinkThread=Class(TThread)
  Protected
    Procedure Execute; Override;


End;


var
  SplashCustomEvent: TSplashCustomEvent;


implementation

uses SysUtils, Graphics, Windows, ASFDemoMain, ShellAPI, Math;



{ TClickLinkThread }

Procedure TClickLinkThread.Execute;
{var
  Rlst: LongBool;
  StartUpInfo: TStartUpInfo;
  ProcessInfo: TProcessInformation;
  DBP:string;
///////////////
  function GetDefaultBrowserPath: string;
  begin
    Result := '';
    with TRegistry.Create do begin
      try
        RootKey := HKEY_CLASSES_ROOT;
        if OpenKey('http\shell\open\command', False) then begin
          Result := StringReplace(ReadString(''), '"', '', [rfReplaceAll]);
          CloseKey;
          end;
        end;
      finally
        Free;
      end;
    end;
  end;}
/////////////////
Begin
{  FillChar(StartUpInfo, SizeOf(TStartUpInfo), 0);
  with StartUpInfo do begin
    cb := SizeOf(TStartUpInfo);
    dwFlags := STARTF_USESHOWWINDOW or STARTF_FORCEONFEEDBACK;
    wShowWindow := SW_SHOWNORMAL;
  end;
  DBP := GetDefaultBrowserPath();
  if DBP <> '' then begin
    CreateProcess(PChar(DBP), ' http://bitoff.ru/asf-main', nil, nil, false, NORMAL_PRIORITY_CLASS, nil, nil, StartUpInfo, ProcessInfo);
  end;}
  ShellExecute(0, nil, PChar('http://bitoff.ru/asf-main'), nil, nil, SW_SHOWNORMAL);
End;




{ TSplashCustomEvent}

constructor TSplashCustomEvent.Create;
begin
  FGPImage := TGPBitmap.Create(ExtractFilePath(ParamStr(0)) + 'img\Splashbg.png');
  FStep := 10;
  FPoolLeft := 44;
  FPoolTop := 40;
  FRain := False;
  AboutShow := False;
  Randomize;
  inherited;
end;

destructor TSplashCustomEvent.Destroy;
begin
  inherited;
  if FWater <> nil then
    FWater.Free;
  FGPImage.Free;
end;

procedure TSplashCustomEvent.DestroySplash(Sender: TObject);
begin
  AnimationSplashForm := nil;
end;

procedure TSplashCustomEvent.CustomDraw(Sender: TObject; GPGraphics: TGPGraphics);
var
  X, Y: Integer;
  ASF: TAnimationSplashForm;
begin
  if AnimationSplashForm <> nil then begin
    if FWater = nil then begin
      FWater := TWave.Create;
      FWater.SourceBitmap := TGPBitmap.Create;
      FWater.SourceBitmap := TGPBitmap(FGPImage.Clone);
      FWater.DestBitmap := TGPBitmap.Create;
      FWater.DestBitmap := TGPBitmap(FGPImage.Clone);
      FWater.Init;
    end
    else begin
      Inc(FStep);
      if FRain and ((FStep mod 5) = 0) then begin
        X := Random(FWater.SourceBitmap.GetWidth);
        Y := Random(FWater.SourceBitmap.GetHeight);
        FWater.HeightBlob(X, Y, 2000, 2);
      end;
      FWater.CalcStep;
      FWater.ApplyMaskToBmp;
      GPGraphics.DrawImage(FWater.DestBitmap, FPoolLeft, FPoolTop);
    end;
    ASF := TAnimationSplashForm(Sender);
    if ASF.isWaitMode then
      ASF.TextData.Items[2].Text := 'Wait mode'
    else
      if not AboutShow then
        ASF.TextData.Items[2].Text := FloatToStr(Round(ASF.ProgressBarPercentSection)) + '%' +
            ' (' + FloatToStr(Round(ASF.ProgressBarPercentFull)) + '%)';
  end;
end;

function TSplashCustomEvent.PixelInRect(x, y: integer; aRect: TGPRect): Boolean;
begin
  Result := (X >= aRect.X) and (X <= aRect.Width + aRect.X) and (Y >= aRect.Y) and (Y <= aRect.Height + aRect.Y);
end;

function TSplashCustomEvent.PixelInRect(x, y: integer; aRect: TGPRectF): boolean;
begin
  Result := (X >= aRect.X) and (X <= aRect.Width + aRect.X) and (Y >= aRect.Y) and (Y <= aRect.Height + aRect.Y);
end;

function TSplashCustomEvent.InPool(X, Y: Integer): Boolean;
begin
  Result := (FWater <> nil) and (PixelInRect(X, Y, MakeRect(FPoolLeft, FPoolTop,
                  Int(FGPImage.GetWidth) + FPoolLeft,
                  Int(FGPImage.GetHeight) + FPoolTop)));
end;

procedure TSplashCustomEvent.SetRainTextData(ASF: TAnimationSplashForm);
begin
  if FRain then begin
    ASF.TextData.Items[1].Color := $FFFFFFFF;
    ASF.TextData.Items[1].Text := 'Rain on';
  end
  else begin
    ASF.TextData.Items[1].Color := $FFA9A9A9;
    ASF.TextData.Items[1].Text := 'Rain off';
  end;
end;

procedure TSplashCustomEvent.MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  ASF: TAnimationSplashForm;
begin
  ASF := TAnimationSplashForm(Sender);
  if InPool(X, Y) then begin
    FWater.HeightBlob(X - FPoolLeft, Y - FPoolTop, 500, 5);
  end;
  if PixelInRect(X, Y, ASF.TextData.Items[0].TextBox) then begin  // web site
    ASF.Cursor := crHand;
    ASF.TextData.Items[0].Color := $FFFF0000;
    SetRainTextData(ASF);
  end
  else begin
    if PixelInRect(X, Y, ASF.TextData.Items[1].TextBox) then begin  // Rain
      ASF.Cursor := crHand;
      ASF.TextData.Items[1].Color := $FFFF0000;
      ASF.TextData.Items[0].Color := $FFA9A9A9;
    end
    else begin
      if (PixelInRect(X, Y, ASF.TextData.Items[2].TextBox)) and AboutShow then begin
        ASF.Cursor := crHand;
        ASF.TextData.Items[2].Color := $FFFFFFFF;
      end
      else begin
        ASF.Cursor := crArrow;
        ASF.TextData.Items[2].Color := $FFA9A9A9;
        ASF.TextData.Items[0].Color := $FFA9A9A9;
        SetRainTextData(ASF);
      end;
    end;

  end;
end;

procedure TSplashCustomEvent.MouseDown(Sender: TObject; Button: TMouseButton;
    Shift: TShiftState; X, Y: Integer);
var
  ASF: TAnimationSplashForm;
begin
  if Button = mbLeft then begin
    ASF := TAnimationSplashForm(Sender);
    if InPool(X, Y) then begin
      FWater.HeightBlob(X - FPoolLeft, Y - FPoolTop, 5000, 6);
    end;
    if PixelInRect(X, Y, ASF.TextData.Items[0].TextBox) then begin  // web site
      TClickLinkThread.Create(False);
    end
    else begin
      if PixelInRect(X, Y, ASF.TextData.Items[1].TextBox) then begin  // Rain
        FRain := not FRain;
        SetRainTextData(ASF);
      end
      else begin
        if (PixelInRect(X, Y, ASF.TextData.Items[2].TextBox)) and AboutShow then begin
          ASF.CloseSplash(0);
        end;
      end;
    end;
  end;


end;


initialization
  SplashCustomEvent := TSplashCustomEvent.Create;

finalization
  SplashCustomEvent.Free;

end.
