unit WaveEng;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  GDIPOBJ, GDIPAPI, GDIPUTIL;

type
  TWave = class
  private
    FZ: array[0..1] of array of array of Integer;
    FStep: Byte;
    FDestBitmap: TGPBitmap;
    FSourceBitmap: TGPBitmap;
    function GetHeight: Integer;
    function GetWidth: Integer;
    function GetZ(x, y: Integer): Integer;
    procedure SetZ(x, y: Integer; const Value: Integer);
  public
    procedure Init;
    procedure CalcStep;
    procedure ApplyMaskToBmp;
    procedure HeightBlob(x, y, Power, Radius: Integer);
    constructor Create;
    destructor Destroy; override;

    property SourceBitmap: TGPBitmap read FSourceBitmap write FSourceBitmap;
    property DestBitmap: TGPBitmap read FDestBitmap write FDestBitmap;
    property Width: Integer read GetWidth;
    property Height: Integer read GetHeight;
    property Z[x, y: Integer]: Integer read GetZ write SetZ;
  end;

implementation
{ TWave }

procedure TWave.ApplyMaskToBmp;
var
  x, y: Integer;
  p: pByteArray;
  SBitmapData, DBitmapData: TBitmapData;

  function GetZ(N: Byte; x, y: Integer): Integer;
  begin
    Result := 0;
    case N of
      1: begin
          if (x > 0) and (y > 0) then Result := FZ[FStep][x - 1][y - 1];
        end;
      2: begin
          if y > 0 then Result := FZ[FStep][x][y - 1];
        end;
      3: begin
          if (x < Width - 1) and (y > 0) then Result := FZ[FStep][x + 1][y - 1];
        end;
      4: begin
          if x < Width - 1 then Result := FZ[FStep][x + 1][y];
        end;
      5: begin
          if (x < Width - 1) and (y < Height - 1) then Result := FZ[FStep][x + 1][y + 1];
        end;
      6: begin
          if y < Height - 1 then Result := FZ[FStep][x][y + 1];
        end;
      7: begin
          if (x > 0) and (y < Height - 1) then Result := FZ[FStep][x - 1][y + 1];
        end;
      8: begin
          if x > 0 then Result := FZ[FStep][x - 1][y];
        end;
    end;
  end;

  procedure GetColor(x2, y2: Integer);
  var
    _x, _y: Integer;
    p2: pByteArray;
  begin
    _x := x2 + (Z[x2, y2] - GetZ(4, x2, y2)) div 8;
    _y := y2 + (Z[x2, y2] - GetZ(2, x2, y2)) div 8;
    if (_x < 0) or (_y < 0) or (_x >= Width) or (_y >= Height) then begin
      p2 := Pointer(Integer(SBitmapData.Scan0) + Width * y2 * 4);
      p[x] := p2[x];
      p[x + 1] := p2[x + 1];
      p[x + 2] := p2[x + 2];
    end
    else begin
      p2 := Pointer(Integer(SBitmapData.Scan0) + Width * _y * 4);
      p[x] := p2[_x * 4];
      p[x + 1] := p2[_x * 4 + 1];
      p[x + 2] := p2[_x * 4 + 2];
    end;

  end;
begin

  FDestBitmap.LockBits(MakeRect(0, 0, Width, Height), ImageLockModeRead or ImageLockModeWrite, PixelFormat32bppARGB, DBitmapData);
  FSourceBitmap.LockBits(MakeRect(0, 0, Width, Height), ImageLockModeRead or ImageLockModeWrite, PixelFormat32bppARGB, SBitmapData);
  try
    for y := 0 to Height - 1 do begin
      p := Pointer(Integer(DBitmapData.Scan0) + Width * y * 4);
      x := 0;
      while x < Width * 4 do begin
        GetColor(x div 4, y);
        Inc(x, 4);
      end;
    end;
  finally
    FDestBitmap.UnlockBits(DBitmapData);
    FSourceBitmap.UnlockBits(SBitmapData);
  end;
end;

procedure TWave.CalcStep;
var
  _Step: Byte;
  x, y, _t: Integer;
  function GetZ(N: Byte; x, y: Integer): Integer;
  begin
    Result := 0;
    case N of
      1: begin
          if (x > 0) and (y > 0) then Result := FZ[FStep][x - 1][y - 1];
        end;
      2: begin
          if y > 0 then Result := FZ[FStep][x][y - 1];
        end;
      3: begin
          if (x < Width - 1) and (y > 0) then Result := FZ[FStep][x + 1][y - 1];
        end;
      4: begin
          if x < Width - 1 then Result := FZ[FStep][x + 1][y];
        end;
      5: begin
          if (x < Width - 1) and (y < Height - 1) then Result := FZ[FStep][x + 1][y + 1];
        end;
      6: begin
          if y < Height - 1 then Result := FZ[FStep][x][y + 1];
        end;
      7: begin
          if (x > 0) and (y < Height - 1) then Result := FZ[FStep][x - 1][y + 1];
        end;
      8: begin
          if x > 0 then Result := FZ[FStep][x - 1][y];
        end;
    end;
  end;
begin
  Inc(FStep);
  if FStep >= 2 then FStep := 0;
  if FStep = 0 then _Step := 1 else _Step := 0;
  for x := 0 to Width - 1 do
    for y := 0 to Height - 1 do begin
      _t := (GetZ(1, x, y) + GetZ(2, x, y) + GetZ(3, x, y) +
        GetZ(4, x, y) + GetZ(5, x, y) + GetZ(6, x, y) +
        GetZ(7, x, y) + GetZ(8, x, y)) div 4 - FZ[_Step][x][y];
      FZ[_Step][x][y] := _t - _t div 8;
    end;
end;

function TWave.GetHeight: Integer;
begin
  Result := FSourceBitmap.GetHeight;
end;

function TWave.GetWidth: Integer;
begin
  Result := FSourceBitmap.GetWidth;
end;

function TWave.GetZ(x, y: Integer): Integer;
begin
  Result := FZ[FStep][x][y];
end;

procedure TWave.HeightBlob(x, y, Power, Radius: Integer);
var
  _x, _y, d: Integer;
begin
  d := Radius div 2;
  for _x := x - d to x + d do
    for _y := y - d to y + d do begin
      if (_x >= 0) and (_x < Width) and (_y >= 0) and (_y < Height) then Z[_x, _y] := Power;
    end;
end;

procedure TWave.Init;
var
  x, y, i: Integer;
begin
  for i := 0 to 1 do begin
    SetLength(FZ[i], FSourceBitmap.GetWidth);
    for x := 0 to FSourceBitmap.GetWidth - 1 do SetLength(FZ[i][x], FSourceBitmap.GetHeight);
    for x := 0 to FSourceBitmap.GetWidth - 1 do
      for y := 0 to FSourceBitmap.GetHeight - 1 do FZ[i][x][y] := 0;
  end;
  FStep := 0;
end;

procedure TWave.SetZ(x, y: Integer; const Value: Integer);
begin
  FZ[FStep][x][y] := Value;
end;

constructor TWave.Create;
begin
{  FSourceBitmap := TGPBitmap.Create(FileName);}
{  FSourceBitmap.SetResolution(72, 72);}
{  FDestBitmap := TGPBitmap.Create();}
{  FSourceBitmap.SetResolution(72, 72);}
{  FDestBitmap := TGPBitmap(FSourceBitmap.Clone);}
end;

destructor TWave.Destroy;
begin
  FSourceBitmap.Free;
  FDestBitmap.Free;
end;

end.

